=begin
**
** knowledge.rb
** 1/AUG/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   This object acts a local cache of the remote ActiveResources. Everytime a
** new revision is detected this cache is refreshed with copies of the remote
** objects.
** 
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  module Model
    # This object acts a local cache of the remote ActiveResources. Everytime a
    # new revision is detected this cache is refreshed with copies of the remote
    # objects.
    class Knowledge
      attr_accessor :revision, :nodes, :categories
      
      #===================================================== public methods
      public
      
      # Initialise the knowledge base. The revision is set to 0.
      def initialize()
        @revision = 0
      end
      
      #----------------------------------------------------- attr setters
      def categories=(category_list)
        @sorted_categories = {}
        @categories = category_list
      end
      def nodes=(node_list)
        @sorted_nodes = {}
        @nodes = node_list
      end
      
      #----------------------------------------------------- support methods
      
      # For a given +parent_id+, this method returns an array containing all 
      # the direct children of the node whose id is +parent_id+
      def node_children(parent_id)
        # FIXME: raname to children_of(parent_id)
        children = []
        @nodes.each do |n|
          children << n if n.parent_id == parent_id
        end
        return children
      end
      
      # For a given +category_id+ this method find the Category object in the
      # local cache.
      def category_find(category_id)
        if @sorted_categories.size.zero?
          @categories.each do |category|
            @sorted_categories[category.id] = category
          end
        end
        return @sorted_categories.fetch(category_id, Category.new(:name => 'error, this note does not have a category in the server!!'))
      end

      # For a given +node_id+ this method find the Node object in the
      # local cache.
      def node_find(node_id)
        if @sorted_nodes.size.zero?
          @nodes.each do |node|
            @sorted_nodes[node.id] = node
          end
        end
        return @sorted_nodes.fetch(node_id, Node.new(:label => 'error, this node does not exist in the server!!'))
      end      
    end
  end
end

