=begin
**
** support.rb
** 31/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  This file contains support model classes that the client uses on top of the
** active_resources provided by the server.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  module Model
    # This module contains support model classes that the client uses on top of the
    # active_resources provided by the server. These clases include:
    #
    # * Property: each node property as defined in the universe.xml results in 
    # an instance of a Property class being created.
    module Support

      # Each property defined for a node inside the universe.xml will be loaded into 
      # an instance of this class. There are two types of properties: simple 
      # (Property) and multiple choice (MultipleChoiceProperty).
      #
      # Property objects store the name, type and value of a property. They can be
      # serialized to_xml.
      class Property
        attr_accessor :key, :name, :value, :type, :populator
  
        def initialize(key, name, value, populator=nil)
          @key = key
          @name = name
          @value = value
          @populator = populator
        end
  
        def to_xml(options = {})
          xml = options[:builder] ||= Builder::XmlMarkup.new(:indent => options[:indent])
          xml.instruct! unless options[:skip_instruct]
          xml.property { 
            xml.key @key.to_s
            xml.name @name 
            xml.type @value.class.to_s
            xml.value @value
          }
        end
        
        def populator_defined?
          return !@populator.nil?
        end
        def populator
          # If the populator has not been instantiated, do so now. There is no
          # problem if the populator is nil, instance_of? evaluates to +false+
          if @populator.instance_of?(String)
            begin
              @populator = eval(@populator).new
            rescue
              $logger.error{ "There was an error instantiating #{@populator}: #{$!}" }
              $logger.debug{ $@ }
              @populator = nil
            end
          end
          
          return @populator
        end
      end

      class MultipleChoiceProperty < Property
        attr_accessor :options, :multiple
  
        def initialize(key, name, value, options, multiple)
          new_value = value.instance_of?(String) ? [value] : value
          super(key, name, new_value)
          @options = options
          @multiple = multiple
        end
  
        def to_xml(options = {})
          xml = options[:builder] ||= Builder::XmlMarkup.new(:indent => options[:indent])
          xml.instruct! unless options[:skip_instruct]
          xml.multiplechoiceproperty { 
            xml.key @key.to_s
            xml.name @name 
            xml.multiple @multiple
            xml.options {
              @options.each { |o| xml.option o }
            }
            xml.values {
              @value.each { |v| xml.value v  }
            }
      
          }
        end  
      end
      
    end # Support
  end # Model
end # Core