=begin
**
** builder.rb
** 05/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Multiverse builder is the class that keeps the Multiverse under control.
** Lists, finds, enables, disables universes.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  module Providers
    module Multiverse
      class Builder
        include Interfaces::Builder
      
        #===================================================== private methods
        private
        def find_universes(path='./')
          # don't go any further if the multiverse directory doesn't exist
          if not File.exists?(path)
            raise Core::Exceptions::Multiverse::MultiversePathNotFoundException.new("Multiverse path does not exist: [#{path}]") 
          end
        
          # add a trailing slash
          path += '/' unless (path =~ /\/$/)
        
          list = []
          Dir.entries(path).each do |universe|
            next if universe =~ /\./
          
            universepath = path + universe
            universefile = File.expand_path(universepath + '/universe.xml')
            if File.directory?(universepath) && File.exists?(universefile)
              $logger.debug { "\t\tAdding universe: #{universe}" }
              begin
                list << @parser.parse( :file => universefile )
              rescue Core::Exceptions::Multiverse::UniverseParsingException
                $logger.error { $! }
              end
            end
          end
          return list
        end
      
        #===================================================== public methods
        public      
        def initialize(params={})
          @parser = Parser.new()
          @multiverse = find_universes( params.fetch(:path, 'multiverse/') )
        end
      end
    end
  end
end
