=begin
**
** extended_node.rb
** 4/AUG/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   This class extends the standard node to the extra layer of information
** that the node has because of its belonging to a universe (i.e. universe 
** uid, node type, etc.).
**  
** 
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'xmlsimple'

module Core
  module Providers
    module Multiverse
      module Node
        # This module extends the standard node to the extra layer of information
        # that the node has because of its belonging to a universe (i.e. universe 
        # uid, node type, etc.).
        module Extensions
          attr_accessor :properties_id, :universe_uid, :node_type
          
          #===================================================== private methods
          private 
          
          def from_xml(text)
            begin
              info = XmlSimple.xml_in( text, 'SuppressEmpty' => '' )
            rescue 
              # if there is an error loading the XML
              $logger.error{ "Error loading properties of node #{self.id}" } 
              @properties = []
              return
            end
            
            # ----------------------------------- uuid & node type
            @universe_uid = info['uid'].first
            @node_type = info['type'].first['content'].to_i
            
            # ----------------------------------- properties
            out = []
            properties = info['properties'].first
            properties.fetch( 'property', []).each do |prop|
              value = prop['value'].first

              case prop['type'].to_s
              when 'Fixnum'
                value = value.to_i
              when 'TrueClass', 'FalseClass'
                value = (value == 'true') ? true : false
              end
              out << Property.new( prop['key'].first, prop['name'].first, value)
            end

            properties.fetch( 'multiplechoiceproperty', []).each do |prop|
              out << MultipleChoiceProperty.new( prop['key'].first, 
                                                 prop['name'].first, 
                                                 prop['values'].first['value'],
                                                 prop['options'].first['option'],
                                                 prop['multiple'].first == "true" ? true : false  )
            end
            @properties = out
          end
          
          # Find the Note that contains the node extended information and
          # parse it.
          def find_info
            @properties = []
            @attributes['notes'].each_index do |i|
              note = @attributes['notes'][i]
              if note.category_id == $config.get_option(:multiverse_category_id).to_i
                self.properties_id = note.id
                from_xml(note.text)
                $logger.debug{ "#{@properties.size} properties found for #{label}"}
                @attributes['notes'].delete_at(i)
                break
              end
            end
            
            if @properties.size.zero?
              $logger.debug{ "No properties note found for node: #{self.label}"}
            end
          end
          
          #===================================================== public methods
          public 
          def notes
            find_info if @properties.nil?
            @attributes['notes']
          end
          def properties
            find_info if @properties.nil?
            @properties
          end
          
          def universe_uid
            find_info if @universe_uid.nil?
            @universe_uid
          end
          def node_type
            find_info if @node_type.nil?
            @node_type
          end

        end
      end
    end
  end
end

module Core
  module Model
    module Resources
      class Node #:nodoc:
        include Core::Providers::Multiverse::Node::Extensions
      end
    end
  end
end
