=begin
**
** meta_section_parser.rb
** 04/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Parser for the <meta> section of the XML document. 
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  module Providers
    module Multiverse
      module Parsers
        # Parser for the +meta+ section of the XML document. The meta section
        # needs to contain at least a universe +name+ and +uid+. Their presence
        # and validity is checked in the check(document) method.
        class MetaSectionParser
          include Interfaces::Parsers::SectionParser
        
          # Check that the required +meta+ tags are present in the document.
          def check(document) 
            if ( document.root.elements['meta/name'].nil?  )
              raise Core::Exceptions::Multiverse::UniverseParsingException.new('Required <name> tag not found inside the <meta> section.')
            end
            if ( document.root.elements['meta/uid'].nil? )
              raise Core::Exceptions::Multiverse::UniverseParsingException.new('Required <uid> tag not found inside the <meta> section.') 
            end
            if ( document.root.elements['meta/uid'].text.nil? )
              raise Core::Exceptions::Multiverse::UniverseParsingException.new('<uid> value is not meaningful.') 
            end

            if ( document.root.elements['meta/creator'] )
              creatortag = document.root.elements['meta/creator']
              if ( creatortag.attributes['name'].nil? && creatortag.attributes['email'].nil?  )
                raise Core::Exceptions::Multiverse::UniverseParsingException.new('<creator> tag needs at least a name or an email attribute.') 
              end
            end
          
          end
                  
          # Parse the +meta+ section of the document and return a hash containing
          # all the information
          def parse(document, universe) 
            universe.meta = {}
            metaroot = document.root.elements['meta']
            metaroot.elements.each do |m|
              key = m.name.to_sym
              if (:creator == key)
                universe.meta[key] = {}
                m.attributes.each do |name, value|
                  universe.meta[key][name.to_sym] = value
                end
              else
                universe.meta[key] = m.text
              end
            end
          end
        end
      end
    end
  end
end