=begin
**
** nodes_section_parser.rb
** 04/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Parser for the <nodes> section of the XML document. 
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  module Providers
    module Multiverse
      module Parsers
        # Parser for the +nodes+ section of the XML document. 
        class NodesSectionParser
          include Interfaces::Parsers::SectionParser

          #===================================================== private methods
          private
          
          def find_populator_for(property, universe_name, universe_path=nil)
            # Look for populators inside the universe path
            if universe_path
              $logger.debug{ "\t\t\tFinding populators for '#{property.name}'" }
              populator = "#{universe_path}/populators/#{property.key}_populator.rb"
              if File.exists?(populator)
                begin
                  klass = "#{universe_name}::Populators::#{File.basename(populator,'.rb').camelize}"
                  load populator
                  begin
                    eval(klass)
                    return klass
                  rescue
                    $logger.warn{ "\t\t\tWarning, after loading #{File.basename(populator)} no #{klass} was defined" }
                    $logger.debug{ $! }
                  end
                rescue
                  $logger.error{ "\t\t\tThere was an error loading the populator for '#{property.name}': #{$!}" }
                  $logger.debug{ $! }
                end
              end
              return nil
            end
            
          end
          
          def parse_property(property_hash)
            property = nil
            case property_hash['type'].downcase
            when 'string'
              property = Property.new( property_hash['key'], property_hash['name'], '')
            when 'boolean'
              property = Property.new( property_hash['key'], property_hash['name'], false)
            end                      
            return property
          end

          #===================================================== public methods
          public
        
          # Check that the required +nodes+ tags are present in the document.
          def check(document) 
            ; # no requirements for the <nodes> section
          end
                  
          # Parse the +nodes+ section of the document and return an array 
          # containing all the information
          def parse(document, universe)        
            universe.nodes = []
            nodesroot = document.root.elements['nodes']
            nodesroot.elements.each do |n|
              # TODO: validate node
              node = { 
                :name => n.attributes['name'], 
                :type => n.attributes['type'].to_i,
                :properties => {}
              }
              if n.elements['properties'] && !n.elements['properties'].size.zero?
                n.elements['properties'].elements.each do |prop|
                  property = parse_property(prop.attributes)
                  property.populator = find_populator_for(property, universe.meta[:name], universe.path)
                  if not property.nil?
                    node[:properties][property.key] = property
                  end
                end
              end
              universe.nodes << node
            end
          end
        end
      end
    end
  end
end
