=begin
**
** add.rb
** 11 February 2009
** Siebert Lubbe siebertlubbe [ at ] googlemail [ dot ] com
**
** Desc:
** Implements the add functions that extends to adding nodes and categories
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Extensions
  # This module provides a methods tp add nodes or categories
  # Actions:
  # [+node+] adds a node to the node tree
  # [+category+] adds a note category
  class Add < Extensions::Namespace


    #----------------------------------------------------- private methods
    private

    #----------------------------------------------------- Dispatcher implementation
    INFO = {
      :namespace => 'add',
      :description => 'add elements to the project',
      :commands => {
        'node' => {
          :desc => 'Adds a node to the node tree.',
          :syntax => [
          {
            :required => true,
            :label => 'parent_id',
            :desc => 'The id of parent to the new node. Use 0 if it is to be a root node.',
            :regexp => /^\d*$/
          },
          {
            :required => true,
            :label => 'label',
            :desc => 'The label/name of the new node.',
            :regexp => /^\w[\w\s]*$/
          }
          ]
        },
        'note' => {
          :desc => 'Adds a note to a node in the tree.',
          :syntax => [
          {
            :required => true,
            :label => 'node_id',
            :desc => 'The id of the node to which the note will be added.',
            :regexp => /^\d*$/
          },
          {
            :required => true,
            :label => 'name',
            :desc => 'user name',
            :regexp => /^\w[\w\s]*$/
          },
          {
            :required => true,
            :label => 'category_id',
            :desc => 'The id of the category to which the note will be added.',
            :regexp => /^\d*$/
          },
          {
            :required => true,
            :label => 'contents',
            :desc => 'The content of the note.',
            :regexp => /.*/ 
          }
          ]
        },
        'category' => {
          :desc => 'Adds a note category.',
          :syntax => [
          {
            :required => true,
            :label => 'name',
            :desc => 'name of the new category',
            :regexp => /^\w[\w\s]*$/
          }
          ]
        }
      }
    }

    # Implements the add node fucntionality. See the above for allowed parameters
    def node(*args)
      # the parent_id needs to be an integer or a nil value
      args[1] = args[1] == '0' ? nil : args[1].to_i
      # call the service provider to add a node with the specified parameters
      @controller.request_service(:model_add_node, args[1], nil, args[2])
      return "Added node: #{args[2]}"
    end

    # Implements the add note fucntionality. See the above for allowed parameters
    def note(*args)
      # the parent_id needs to be an integer value
      args[1] = args[1].to_i
      # the category_id needs to be an integer value
      args[3] = args[3].to_i
      # the contents of the note should include all the rest of the args[]
      content = ""
      4.upto(args.length-1) do |index|
        content += args[index]+" "
      end
      # call the service provider to add a note with the specified parameters
      @controller.request_service(:model_add_note, args[1], args[2] , args[3], content)
      return "Added note to node with id : #{args[1]}"
    end

    # Implements the add category fucntionality. See the above for allowed parameters
    def category(*args)
      # call the service provider to add a category with the specified parameters
      @controller.request_service(:model_add_category, args[1])
      return "Added category: #{args[1]}"
    end

  end

end
