#!/usr/bin/ruby
#
# encode_module.rb
#
# john [ at ] tinternet.org.uk
#
# 2007-11-12
#
# v1.1
#
# Module for dradis to encode strings using various different encoding
# schemes
#
# TODO:
# * md5 lookup table (work in progress)
#
# TODO (when Encoder module supports it)
# * add unicode support
# * bare byte encoding support
# * decode strings
#

require "./extensions/encoder/Encoder.rb"
require "./extensions/lookup/lookup.rb"

module Extensions
  class Encode < Extensions::Simple
    include Encoder
    include Lookup
    
    #----------------------------------------------------- private methods
    private
    #----------------------------------------------------- NamespaceDispatcher interface  
    INFO = { 
      :commands => {
        'encode' => { 
          :desc => 'Encodes a string using various differnet encoding schemes', 
          :syntax => [ '> encode [option] <string to encode>' ]
        },
        'lookup' => { 
          :desc => 'Basically greps a file, useful for lookup tables', 
          :syntax => [ '> lookup <string> <file-path>' ]
        },
        'decode' => { 
          :desc => 'Decodes a string into the specified encoding type', 
          :syntax => [ '> decode [-t <type of encoding>] string' ]
        }
      }
    }

    # display help for the user
    def printHelpEncode
        return "\nuseage:\n\tencode [option] text\n\n Possible options (leave blank to encode to all schemes):\n
\t-b\tbinary
\t-uri\turi encoding
\t-hex\thex encoding
\t-utf7\tUTF-7 encoding
\t-dp\tdouble percent hex encoding
\t-dn\tdouble nibble hex encoding
\t-1n\tfirst nibble hex encoding
\t-2n\tsecond nibble hex encoding
\t-pu\tpercetn U encoding
\t-md5\tMD5
\t-md5x2\tdouble md5
\t-sha\tSHA1
\t-shax2\tdouble sha1
\t-b64\tbase64\n"
    end

    def printHelpLookup
        return "\nuseage:\n\tlookup <text> <file-path>\n\n"
    end

    # Encode a string - currently will encode to all encoding types which will
    # always be the default but when i get round to it there will be an option to 
    # specify the encoding scheme to use
    def encode(*args)

      # if only one argument then print help and finish
      if args.length == 1
        return printHelpEncode
      end


      # first of all check the length of the args array to see if
      # an argument has been passed. if so perform only the encoding
      # specified
      if args.length > 2
        option = String.new(args[1].to_s)

        # dradis does not currently allow argumetns containing spaces so in order
        # to facilitate this i need to grab the rest of the elements in the array
        # and assume them to be one long string, preserving spaces - each element 
        # in the array should be followed by a space
        args.delete_at(0)
        args.delete_at(0) # remember once you delete 0 what was 1 becomes 0 ;)

        # currently spaces are not handled well by dradis so i have to assume that
        # each argument is seperated by a space. Leading and trailing white space 
        # will be lost. In order to facilitate leading and trailing white space you
        # must enclose your string (probably in " or ') however these will also be
        # encoded an you will have to manually strip them out.
        args.collect! { |element| element + " " }
        args[args.length-1].rstrip!
        string = args.to_s

        case option
        when "-b"
          return string+":\n"+toBinary(string)
        when "-uri"
          return string+":\n"+toURI(string)
        when "-hex"
          return string+":\n"+toHex(string)
        when "-utf7"
          return string+":\n"+toUTF7(string)
        when "-dp"
          return string+":\n"+toDoublePercentHex(string)
        when "-dn"
          return string+":\n"+toDoubleNibbleHex(string)
        when "-1n"
          return string+":\n"+toFirstNibbleHex(string)
        when "-2n"
          return string+":\n"+toSecondNibbleHex(string)
        when "-pu"
          return string+":\n"+toPercentU(string)
        when "-md5"
          return string+":\n"+toMd5(string)
        when "-md5x2"
          return string+":\n"+toDoubleMd5(string)
        when "-sha"
          return string+":\n"+toSHA1(string)
        when "-shax2"
          return string+":\n"+toDoubleSHA1(string)
        when "-b64"
          return string+":\n"+toBase64(string)
        else
          # if we get here then the user may well have been trying to encode
          # a couple of words seperated by a space rather than specifying an
          # argument. In which case add what we thought was an argument back 
          # to the string and encode that in all schemes
          return encodeAll(option+" "+string)
        end

      else
        # looks like no arguments specified so encode entire string by deleting
        # first element in args array and turning all remaining elemnts back into a
        # string and adding spaces between each.
        #
        # currently spaces are not handled well by dradis so i have to assume that
        # each argument is seperated by a space. Leading and trailing white space 
        # will be lost. In order to facilitate leading and trailing white space you
        # must enclose your string (probably in " or ') however these will also be
        # encoded an you will have to manually strip them out.
        args.delete_at(0)
        args.collect! { |element| element + " " }
        args[args.length-1].rstrip!
        string = args.to_s

        return encodeAll(string)
      end
    end

    # searches a text file for an md5 hash
    def lookup(*args)

      if args.length == 3
        return Lookup.search(args[1], args[2])
      else
        return printHelpLookup
      end
    end

    # method to decode a string. if no type is specified then it will try and decode
    # all types of encoding and if matching the correct format will also perform lookups
    # against md5 and sha hashes (obviously less efficient than specifying a type)
    def decode(*args)

      return "Not yet implemented... coming soon!"

    end


    public
  end


end
