#!/usr/bin/ruby
#
# encoder.rb
#
# john [ at ] tinternet.org.uk
#
# 2007-11-12
# 2008-10-29 - added utf7 encoding functionality
#
# v1.1
#
# Module for to encode strings using various different encoding
# schemes
#
# TODO:
#       * add UTF-8 encoding
#       * add bare byte encoding
#       * create a decoder
#


require 'digest/md5'
require 'digest/sha1'
require 'uri'
require 'base64'

module Encoder
  # binary
  def toBinary(string)

    binary = ""
    string.scan(/./) do |char|
      binary+=char.unpack("B8").to_s
    end

    return binary.rstrip

  end

  def toURI(string)
    URI.escape(string)
  end

  # hex encoding
  def toHex(string)

    hex = ""
    string.scan(/./) do |char|
      hex+="%#{char.unpack("H4").to_s}"
    end

    return hex.rstrip

  end

  # double percent hex encoding
  def toDoublePercentHex(string)

    hex = ""
    string.scan(/./) do |char|
      hex+="%25#{char.unpack("H4").to_s}"
    end

    return hex.rstrip

  end



  # converts a single char into UTF-7
  #
  # TODO: needs fully testing - this was all a bit of a hack
  #
  # @ = +AEA-
  # < = +ADw-
  # & = +ACY-
  #
  def toUTF7char(char)
    s_bin8 = toBinary(char)
    s_bin16 = s_bin8.rjust(16,'0')
    a_bin6 = s_bin16.scan(/.{6}/)

    # calculate how many string elements are not in the array
    missing = ((s_bin16.length) - (a_bin6.length * 6))
    a_bin6.push s_bin16[(s_bin16.length-missing),(s_bin16.length)]

    # ensure that the last elements in the array are padded out with 0's to make 6 chars
    a_bin6[a_bin6.length-1] = a_bin6[a_bin6.length-1].ljust(6,'0')

    mainstring = ""

    a_bin6.each do |element|
      tmparr = [element]

      # TODO: figure out what is going on here
      # really i only need to encode certain characters - it may be better to just do a lookup
      # assuming it is not nothing then base64 encode
      if element.to_s != '000000'
        mainstring.concat(toBase64(tmparr.pack("B6")))
      end
    end

    puts mainstring.inspect

    mainstring.gsub!("A==\n","")

    puts a_bin6.inspect
    puts "a_bin6.length = #{a_bin6.length}"

    if a_bin6[0] == '000000'
      mainstring = "A"+mainstring
    end
    if a_bin6[a_bin6.length-1] == '000000'
      mainstring.concat('A')
    end

    return "+"+mainstring+"-"
  end


  def toUTF7(string)

    endstring = ""
    string.split(//).each do |char|
      endstring.concat(toUTF7char(char))
    end

    return endstring

  end


  # double nibble hex encoding
  def toDoubleNibbleHex(string)

    hex = ""
    string.scan(/./) do |char|
      strChar = char[0].chr.to_s.unpack("H2")
      hex+="%"
      hex+="%#{strChar[0][0].chr.unpack("H2")}"
      hex+="%#{strChar[0][1].chr.unpack("H2")}"
    end

    return hex.rstrip

  end

  # first nibble hex encoding
  def toFirstNibbleHex(string)

    hex = ""
    string.scan(/./) do |char|
      strChar = char[0].chr.to_s.unpack("H2")[0]
      hex+="%"
      hex+="%#{strChar[0].chr.unpack("H2")}"
      hex+="#{strChar[1].chr.to_s}"
    end

    return hex.rstrip

  end

  # second nibble hex encoding
  def toSecondNibbleHex(string)

    hex = ""
    string.scan(/./) do |char|
      strChar = char[0].chr.to_s.unpack("H2")[0]
      hex+="%"
      hex+="#{strChar[0].chr.to_s}"
      hex+="%#{strChar[1].chr.unpack("H2")}"
    end

    return hex.rstrip

  end

  # Microsoft percent U encoding
  def toPercentU(string)

    hex = ""
    string.scan(/./) do |char|
      hex+="%U00#{char.unpack("H4").to_s}"
    end

    return hex.rstrip

  end

  def toMd5(string)
    return Digest::MD5.hexdigest(string)
  end

  def toDoubleMd5(string)
    return Digest::MD5.hexdigest(Digest::MD5.hexdigest(string))
  end

  def toSHA1(string)
    return Digest::SHA1.hexdigest(string)
  end

  def toDoubleSHA1(string)
    return Digest::SHA1.hexdigest(Digest::SHA1.hexdigest(string))
  end

  def toBase64(string)
    return Base64.encode64(string)
  end

  def encodeAll(mystring)
    result = []
    result << "Hex:\t\t#{toHex(mystring)}"
    result << "URI:\t\t#{toURI(mystring)}"
    result << "UTF-7:\t\t#{toUTF7(mystring)}"
    result << "Double Percent:\t#{toDoublePercentHex(mystring)}"
    result << "Double Nibble:\t#{toDoubleNibbleHex(mystring)}"
    result << "First Nibble:\t#{toFirstNibbleHex(mystring)}"
    result << "Second Nibble:\t#{toSecondNibbleHex(mystring)}"
    result << "Percent U:\t#{toPercentU(mystring)}"
    result << "Binary:\t\t#{toBinary(mystring)}"
    result << "MD5:\t\t#{toMd5(mystring)}"
    result << "Double MD5:\t#{toMd5(toMd5(mystring))}"
    result << "SHA1:\t\t#{toSHA1(mystring)}"
    result << "DoubleSHA1:\t#{toSHA1(toSHA1(mystring))}"
    result << "Base64:\t\t#{toBase64(mystring)}"
    return result
  end

end #end module
