=begin
**
** export.rb
** 2 February 2008
** Siebert Lubbe siebertlubbe@googlemail.com
**
** Desc:
** Exports the Dradis knowledge base to a specified file and format
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Extensions 
  # The class presents methods to export the dradis knowledge base data to the client
  # Actions:
  # [+xml+] exports the data to xml format and save it to a file as requested by the user
  class Export < Extensions::Namespace
    require 'rexml/document'
    include REXML
    #include SoapDriver
    
    #----------------------------------------------------- private methods
    private 
    
    #----------------------------------------------------- Dispatcher implementation
    INFO = {
      :namespace => 'export',
      :description => 'export the knowledge base to a local file',
      :commands => {
        'xml' => {
          :desc => 'export to xml',
          :syntax => [
          {
            :required => true,
            :label => 'file name',
            :regexp => /\w+/
          }
          ]
        }
      }
    }
	
    
    # Export the contents of the DB into an XML document. The structure of
    # the document is:
    #
    # <dradis>
    #   <version></version>
    #   <categories>
    #     <category>
    #       <id></id>
    #       <name></name>
    #     </category>
    #     ...
    #   </categories>
    #   <nodes>
    #     <node>
    #       <label></label>
    #       <notes>
    #         <note>
    #           <category></category>
    #           <text></text>
    #         </note>
    #         ...
    #       </notes>
    #     </node>
    #     ...
    #   </nodes>
    # </dradis>
    #
    def xml(*args)
      # get the knowledge base from the server
      @controller.request_service(:model_knowledge_refresh)
      server_data = @controller.request_service(:model_knowledge_get)
      
      # determine the operating system and extract the directory part of the file name given
      if ::Config::CONFIG["host_os"] == "mswin32"	  
        args[1] =~ /^(.*\\)*(.+)$/
      else
        args[1] =~ /^(.*\/)*(.+)$/	
      end
      
      if !$2 
        return "no file name given"
      end
      
      # if no extention is given then add .xml
      if !(args[1] =~ /.*\..+/)
        args[1] = args[1] + ".xml"
      end
      
      # check if the given file is valid
      if  $1 && !File.exists?($1)
        return "#{$1} is not a valid directory"
      end	  
      if  $1 && !File.stat($1).writable?
        return "you do not have write access to #{$1}"
      end
      
      # instantiate a xml object
      xml_doc = Document.new
      xml_doc << XMLDecl.new

      xml_doc.add_element('dradis')
      xml_doc.root.add_element('version').text = Core::VERSION::STRING 
      xml_doc.root.add_element('revision').text = server_data.revision

      # iterate through the knowledge base data and populate the xml object
      categories = Element.new('categories')
      server_data.categories.each do |category|
        xml_category = Element.new('category')
        xml_category.add_element('id').text = category.id
        xml_category.add_element('name').text = category.name
        categories << xml_category
      end

      xml_doc.root << categories
      nodes = Element.new('nodes')

      server_data.nodes.each do |node|
        xml_node = Element.new('node')
        xml_node.add_element('id').text = node.id
        xml_node.add_element('label').text = node.label

        notes = Element.new('notes')
        node.notes.each do |note|
          xml_note = Element.new('note')
          xml_note.add_element('id').text = note.id
          xml_note.add_element('category').text = note.category_id
          xml_note.add_element('text').text = note.text
          notes << xml_note
        end

        xml_node << notes
        nodes << xml_node
      end

      xml_doc.root << nodes        
             
      # write the xml data to the given file name
      xml_file = File.new(args[1],"w")
      xml_doc.write(xml_file)
      xml_file.close
      
      return "knowledge base data saved to #{args[1]}"      
      
    end

    
  end
  
end
