=begin
**
** controller_test.rb
** 17/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   TestCase for the Controller. Verifies the setup and tear down operations,
** and the communications with View and Service
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'logger'
require 'test/unit'

$: <<  File.dirname(__FILE__) + '/../../'
require 'core/version'
require 'core/exception' 
require 'core/patterns'
require 'core/interfaces'
require 'core/model/knowledge'
require 'core/controller'
require 'ui/console'
# made the core classes available throughout the application
include Core::Model
include Core::Exceptions

class ControllerTest < Test::Unit::TestCase
  def setup
    @ui = Ui::Console::CLI.new()
    $logger = Logger.new(STDOUT)
    $logger.level = Logger::DEBUG
    #$logger.level = Logger::FATAL
  end

  def test_no_or_invalid_view
    # no view
    assert_raises(ArgumentError) do
      controller = Core::Controller.new()
    end
    
    # nil view
    assert_raises(ArgumentError) do
      controller = Core::Controller.new(:view => nil)
    end    
    
    # invalid view
    assert_raises(ArgumentError) do
      controller = Core::Controller.new(:view => Object.new)
    end    
    
    # valid view
    assert_nothing_raised do
      controller = Core::Controller.new(:view => @ui, :service_providers => '../../core/serviceproviders')
    end
  
  end
  
  def test_invalid_service_provider_path
    assert_raises(ServiceProvidersPathNotFoundException) do
      controller = Core::Controller.new(:view => @ui, :service_providers => 'noneexistent')
    end
    assert_nothing_raised do
      controller = Core::Controller.new(:view => @ui, :service_providers => '../../core/serviceproviders')
    end
  end  
  
  def test_detected_service_num
  end
end