=begin
**
** nodeproperties_demo.rb
** 3/AUG/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   TestCase for checking that the Multiverse can pass the properties defined 
** in the universe.xml file to the Interfaceerator.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'logger'
require 'rubygems'
gem 'wxruby', '=1.9.7'
require 'wx'
require 'active_support' # for Array#to_xml

$: <<  File.dirname(__FILE__) + '/../../../'
require 'core/version'
require 'core/exception'
require 'core/patterns'
require 'core/interfaces'
require 'core/model/support'
require 'ui/widgets'
require 'ui/wx/widgets/propertiespane'
$: <<  File.dirname(__FILE__) + '/../../../core/serviceproviders'
require 'multiverse'

include Core
include Exceptions
include Model::Support

class TestApp < Wx::App
  
  def on_node_click(event)
    $logger.debug { "Node clicked: [#{event.get_string}]"}
    @properties.show_properties(1, event.get_client_data)
  end
  
  def save_file
    Dir.mkdir("#{$path}/tempuniverse/") if not File.exists?("#{$path}/tempuniverse/")
    universe = File.new("#{$path}/tempuniverse/universe.xml", 'w+')
    universe.puts @text.get_value
    universe.close
  end
  
  def on_load
    $logger.debug { 'Saving contents of the text box into ./tempuniverse/universe.xml'}
    save_file
    $logger.debug { 'Done. Loading the Multiverse...'}
    provider = Core::Providers::Multiverse::Provider.new(:path => $path)
    $logger.debug { 'Done.'}
    @nodes.clear
    provider.service_multiverse_universe_list.each do |universe|
      next if universe.nodes.size.zero?    
      universe.nodes.each do |node|  
        @nodes.append "#{node[:type]}: #{node[:name]}", node[:properties].values
      end
    end
    
  end
  
  def on_init
    frame = Wx::Frame.new( nil, 
                  Wx::ID_ANY, 
                  "dradis WxWidgets test window",
                  Wx::DEFAULT_POSITION,
                  Wx::Size.new(800, 600) )
    set_top_window(frame)
    
    label = Wx::StaticText.new( frame, Wx::ID_ANY, "Use the text box to paste a universe.xml file. Then click on 'Load'")
    
    @text = Wx::TextCtrl.new( frame, Wx::ID_ANY, '', Wx::DEFAULT_POSITION, Wx::DEFAULT_SIZE, Wx::TE_MULTILINE)
    if not File.exists?("#{$path}/tempuniverse/universe.xml")
      @text.set_value(UNIVERSE_XML)
    else
      @text.set_value(File.read("#{$path}/tempuniverse/universe.xml"))
    end

    @load = Wx::Button.new(frame, Wx::ID_ANY, 'Load')
    evt_button(@load.get_id) { on_load } 
    
    splitter = Wx::SplitterWindow.new(frame, Wx::ID_ANY)
    @nodes = Wx::ListBox.new(splitter, Wx::ID_ANY)
    evt_listbox(@nodes) { |event| on_node_click(event) }
    @properties = Ui::WxWidgets::Widgets::PropertiesPane.new(splitter)
    splitter.split_vertically(@nodes, @properties, 200)
    
    sizer = Wx::BoxSizer.new(Wx::VERTICAL)
    sizer.add label, 0, Wx::ALL, 5
    sizer.add @text, 0, Wx::ALL|Wx::EXPAND, 5
    sizer.add @load, 0, Wx::ALL, 5
    sizer.add splitter, 1, Wx::ALL|Wx::EXPAND, 5
    
    frame.set_sizer_and_fit(sizer)
    frame.show
    return true
  end
  
end

UNIVERSE_XML=<<EOU
<universe>
  <meta>
    <name>Networking</name>
    <uid>baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9</uid>
    <creator email="etd[ at ]nomejortu- dot -com" />
  </meta>
  <etdsoft>
    <!-- this section must be ignored by the parser -->
  </etdsoft>
  <nodes>
    <node type="1" name="Host" />
    <node type="2" name="Protocol" />
  </nodes>      
</universe>
EOU


if $0 == __FILE__
  class Configuration
    def get_option(key)
      return 7
    end
  end
        
  $config = Configuration.new
  $path = File.expand_path('.')
  $logger = Logger.new(STDOUT)
  $logger.level = Logger::DEBUG
  #$logger.level = Logger::FATAL
  TestApp.new.main_loop()
end

