=begin
**
** popupmenu_demo.rb
** 3/AUG/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   TestCase for the popup menu behaviour of the nodes tree. Elements of the 
** popupmenu will be obtained from the Multiverse service provider.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'logger'
require 'rubygems'
gem 'wxruby', '=1.9.7'
require 'wx'
require 'active_support' # for xmlsimple

$: <<  File.dirname(__FILE__) + '/../../../'
require 'core/version'
require 'core/exception'
require 'core/patterns'
require 'core/interfaces'
require 'core/model/support'
require 'ui/widgets'
require 'ui/wx/widgets/propertiespane'
$: <<  File.dirname(__FILE__) + '/../../../core/serviceproviders'
require 'multiverse'

include Core
include Exceptions
include Model::Support

class TestApp < Wx::App
  
  def save_file
    Dir.mkdir("#{$path}/tempuniverse/") if not File.exists?("#{$path}/tempuniverse/")
    universe = File.new("#{$path}/tempuniverse/universe.xml", 'w+')
    universe.puts @text.get_value
    universe.close
  end
  
  def on_popupmenu
    menu = Wx::Menu.new
    addchild = Wx::Menu.new
    
    $logger.debug { 'Saving contents of the text box into ./tempuniverse/universe.xml'}
    save_file
    $logger.debug { 'Done. Loading the Multiverse...'}
    provider = Core::Providers::Multiverse::Provider.new(:path => $path)
    $logger.debug { 'Done.'}
    
    uid = 0
    provider.service_multiverse_universe_list.each do |universe|
      next if universe.nodes.size.zero?      
      universemenu = Wx::Menu.new
      universe.nodes.each do |node|
        uid += 100
        universemenu.append uid, node[:name]
      end
      addchild.append_menu Wx::ID_ANY, universe.meta[:name], universemenu
    end
    menu.append_menu Wx::ID_ANY, 'add child', addchild
    get_top_window().popup_menu menu
  end
  
  def on_init
    frame = Wx::Frame.new( nil, 
                  Wx::ID_ANY, 
                  "dradis WxWidgets test window",
                  Wx::DEFAULT_POSITION,
                  Wx::Size.new(800, 600) )
    set_top_window(frame)
    
    @text = Wx::TextCtrl.new( frame, Wx::ID_ANY, '', Wx::DEFAULT_POSITION, Wx::Size.new(300,300), Wx::TE_MULTILINE)
    if not File.exists?("#{$path}/tempuniverse/universe.xml")
      @text.set_value(UNIVERSE_XML)
    else
      @text.set_value(File.read("#{$path}/tempuniverse/universe.xml"))
    end

    evt_right_down do |event| 
      on_popupmenu
    end
    
    label1 = Wx::StaticText.new( frame, Wx::ID_ANY, "The contents of this text box will be stored in a file in #{$path} and will be loaded by the Multiverse service provider.")
    label2 = Wx::StaticText.new( frame, Wx::ID_ANY, 'Right-click in the text box to see the magic')
    
    sizer = Wx::BoxSizer.new(Wx::VERTICAL)
    sizer.add label1, 0, Wx::ALL, 5
    sizer.add label2, 0, Wx::ALL, 5
    sizer.add @text, 1, Wx::ALL|Wx::EXPAND, 5
    
    frame.set_sizer_and_fit(sizer)
    frame.show
    return true
  end
  
end

UNIVERSE_XML=<<EOU
<universe>
  <meta>
    <name>Networking</name>
    <uid>baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9</uid>
    <creator email="etd[ at ]nomejortu- dot -com" />
  </meta>
  <etdsoft>
    <!-- this section must be ignored by the parser -->
  </etdsoft>
  <nodes>
    <node type="1" name="Host" />
    <node type="2" name="Protocol" />
  </nodes>      
</universe>
EOU

if $0 == __FILE__
  class Configuration
    def get_option(key)
      return 7
    end
  end
        
  $config = Configuration.new
  $path = File.expand_path('.')
  $logger = Logger.new(STDOUT)
  $logger.level = Logger::DEBUG
  #$logger.level = Logger::FATAL
  TestApp.new.main_loop()
end

