=begin
**
** propertiespane_demo.rb
** 29/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   TestCase for the custom Wx widgets.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'rubygems'
gem 'wxruby', '=1.9.7'
require 'wx'
require 'active_support' # for Array#to_xml
require 'xmlsimple'

$: <<  File.dirname(__FILE__) + '/../../../'
require 'core/version'
require 'core/exception'
require 'ui/widgets'
require 'ui/wx/widgets/propertiespane'
require 'core/model/support'

include Core::Model::Support

class TestApp < Wx::App
  
  
  def on_toxml
    @text.set_value( @pane.get_properties().to_xml(:root => 'properties')  )
  end
  def on_fromxml
    out = []
    props = XmlSimple.xml_in( @text.get_value )
    
    props['property'].each do |prop|
      #p prop
      value = prop['value'].first
      
      case prop['type'].to_s
      when 'Fixnum'
        value = value.to_i
      when 'TrueClass', 'FalseClass'
        value = (value == 'true') ? true : false
      end
      out << Property.new( prop['key'].first, prop['name'].first, value)
    end
    
    props['multiplechoiceproperty'].each do |prop|
      #p prop
      out << MultipleChoiceProperty.new( prop['key'].first, 
                                         prop['name'].first, 
                                         prop['values'].first['value'],
                                         prop['options'].first['option'],
                                         prop['multiple'].first == "true" ? true : false  )
    end
    
    #out.each do |prop| p prop end
      
    @pane.show_properties(1, out)
  end

    
  def on_init
    frame = Wx::Frame.new( nil, 
                  Wx::ID_ANY, 
                  "dradis WxWidgets test window",
                  Wx::DEFAULT_POSITION,
                  Wx::Size.new(800, 600) )
    set_top_window(frame)
    
    @pane = Ui::WxWidgets::Widgets::PropertiesPane.new(frame)
    props = 
      [ 
        Property.new( :ip, 'IP address', '10.0.0.1'),
        MultipleChoiceProperty.new( :protocols, 
                                    'IP protocol dialects',
                                    ['ipv4', 'ipv6'],
                                    ['ipv4', 'ipv6'],
                                    true),
        Property.new( :uptime, 'Miliseconds since last reboot', 103421),
        Property.new( :live, 'Is the host alive?', true),
        MultipleChoiceProperty.new( :role, 
                                    'Host role in the network', 
                                    'web server', 
                                    ['web server', 'database server', 'file server'],
                                    false)
      ]
    @pane.show_properties(1, props)

    toxml = Wx::Button.new( frame, Wx::ID_ANY, '-->')
    fromxml = Wx::Button.new( frame, Wx::ID_ANY, '<--')
    @text = Wx::TextCtrl.new( frame, Wx::ID_ANY, '', Wx::DEFAULT_POSITION, Wx::DEFAULT_SIZE, Wx::TE_MULTILINE)

    s_v = Wx::BoxSizer.new(Wx::VERTICAL)
    s_v.add toxml, 0, Wx::ALL, 5
    s_v.add fromxml, 0, Wx::ALL, 5

    sizer = Wx::BoxSizer.new(Wx::HORIZONTAL)
    sizer.add @pane, 1, Wx::ALL|Wx::EXPAND, 5
    sizer.add s_v, 0, Wx::ALL, 5
    sizer.add @text, 1, Wx::ALL|Wx::EXPAND, 5

    frame.set_sizer( sizer )
    frame.show

    evt_button(toxml.get_id) { on_toxml }
    evt_button(fromxml.get_id) { on_fromxml }
    
    return true
  end
  
end


if $0 == __FILE__  
  TestApp.new.main_loop()
end