=begin
**
** console.rb
** 30/JUL/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Simple command line interface that implements the View interface.
** 
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end
require 'irb/input-method'

# Different user interfaces will be grouped under this module.
module Ui
  module Console
    class CLI
      include Core::Interfaces::View
      
      def initialize
        @io = IRB::ReadlineInputMethod.new
        @io.prompt = 'dradis> '
        @controller = nil
        
        trap('SIGINT') do
          signal_handle
        end
        
      end
      
      def signal_handle
        puts "^C detected. Please use: 'quit' if you want to end your dradis session."
      end
      
      def run
        @continue = true
        while @continue
          line = @io.gets

          # handle Ctrl+D
          if @io.eof? 
            puts
            teardown()
            break
          end

          puts @controller.request_service(:parse_command, line.chomp)
        end
      end
    
      def setup(new_controller)
        @controller = new_controller
        @controller.add_observer(:exception, self)
      end
      
      def teardown
        puts 'closing dradis console...'
        @continue = false
      end

      def notify(event, *args)
        case event
          when :exception
            puts "There has been an exception:"
            puts args[1]
        end
      end

    end
  end
end
