=begin
**
** widgets.rb
** 21/JUN/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   This file is used to include all the Qt custom widget used by the GUI.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Ui
  module Qt4
    # Contains all custom Qt widgets created for the application. This file is
    # required by the main GUI to enable the use of custom widgets.
    module Widgets
      
      # Interface to a widget that can be used to browse and display notes for
      # different annotatable object types.
      module NotesBrowserInterface        
        # Invalidate the current view as a result of an action performed by the
        # user (i.e. add a new note, edit the current note, etc.). The widget
        # should display some behaviour indicating that the current view is no
        # longer valid and the user should wait until content is refreshed from
        # the server.
        def invalidate!() raise 'invalidate!() must be overriden'; end
        
        def reload() raise 'reload() must be overriden'; end
        
        def home() raise 'reload() must be overriden'; end
        
        # Given a set of notes (_note_list_) the widge should display them.
        # Other attributes are passed to the widget in order to be returned
        # when the widget signals *editNote* or *addNote*.
        def show_notes(annotatable_type, annotatable_id, note_list) 
          raise 'show_notes(annotatable_type, annotatable_id, note_list)' +
            'must be overriden'; 
        end
      end
    end
  end
end

require 'ui/qt/widgets/commandline'
require 'ui/qt/widgets/notesbrowser'
require 'ui/qt/widgets/chatwidget'

