=begin
**
** chatwidget.rb
** 14/SEP/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Qt widget that connects to an IRC server. It emulates a small irc client.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'Qt4' if $0 == __FILE__

module Ui
  module Qt4
    module Widgets
      private
      class IrcBot < Qt::Widget
        #TODO: fix this
        attr_accessor :browser, :users
        
        slots 'connected()', 'outbound(qint64)', 'inbound()', 'displayError(QAbstractSocket::SocketError)'

        signals 'incoming_message()'
        
        def initialize
          super
          @port = 6667
          @host = 'localhost'
          @nick = 'etd'
            
          @irc = Qt::TcpSocket.new         

          connect(@irc, SIGNAL('connected()'), self, SLOT('connected()'))
          connect(@irc, SIGNAL('bytesWritten(qint64)'),
            self, SLOT('outbound(qint64)'))
          connect(@irc, SIGNAL('readyRead()'), self, SLOT('inbound()'))
          connect(@irc, SIGNAL('error(QAbstractSocket::SocketError)'),
            self, SLOT('displayError(QAbstractSocket::SocketError)'))

        end
        
        def exec
          @irc.abort
          @irc.connectToHost(@host, @port)
        end
        def connected()
          puts '[ircbot] connected to the server'
          @just_logged_in = true
        end
        
        def outbound(size) 
          puts '[ircbot] enviados ' + size.to_s
        end
        def update_user_list(old_nick, new_nick)
          for i in (0..@users.count-1)
            if (n=@users.item(i).text =~ /#{old_nick}/)
              if n == 0
                @users.item(i).text = new_nick
              else
                @users.item(i).text = '@' + new_nick
              end
              break
            end
          end
        end
        
        def handle_server_input(server_message)
          msg = nil
          case server_message.strip
          when /^PING :(.+)$/i
            @browser.append "[ Server ping ]"
            msg = "PONG :#{$1}"
          when /^:(.+?)!(.+?)@(.+?)\sPRIVMSG\s.+\s:[\001]PING (.+)[\001]$/i
            @browser.append "[ CTCP PING from #{$1}!#{$2}@#{$3} ]"
            msg = "NOTICE #{$1} :\001PING #{$4}\001"
          when /^:(.+?)!(.+?)@(.+?)\sPRIVMSG\s.+\s:[\001]VERSION[\001]$/i
            @browser.append "[ CTCP VERSION from #{$1}!#{$2}@#{$3} ]"
            msg = "NOTICE #{$1} :\001VERSION Ruby-irc v0.042\001"
          when /^:(.+?)!(.+?)@(.+?)\sPRIVMSG\s(.+)\s:EVAL (.+)$/i
            @browser.append "[ EVAL #{$5} from #{$1}!#{$2}@#{$3} ]"
            msg = "PRIVMSG #{(($4==@nick)?$1:$4)} :#{$5}"
          when /^:(.+?)!(.+?)@(.+?)\sNICK\s:(.+)$/i
            if $1 == @nick
              @nick = $4
            end
            @browser.append "<b>#{$1}</b> is now known as: #{$4}."
            self.update_user_list($1, $4)
          when /^:(.+?)!(.+?)@(.+?)\sPRIVMSG\s(.+)\s:(.+)$/i
            @browser.append "#{$1}> #{$5}"
          when /^:(.+)!(.+?)@(.+?)\sJOIN\s:(.+)\r\n:(.+)\s(.+)\s(.+)\s@\s(.+)\s:(.+)\r\n:(.+)\s(.+)\s(.+)\s(.+)\s:End\sof\s\/NAMES\slist.$/i
            nicks = $9.split
            nicks.each do |pj| @users.addItem(pj) end
          when /^:(.+?)!(.+?)@(.+?)\sJOIN\s:(.+)$/i
            @users.addItem($1)
            #TODO> :etd!n=etd@5ac0b06d.bb.sky.com QUIT :"Saliendo"
          else
            @browser.append server_message
          end
          return if msg.nil?
          @irc.write(msg, msg.size)
        end
        def inbound()
          msg = @irc.readAll
          #@browser.append "---------\n" + msg.to_s + "\n---------"
          handle_server_input(msg.to_s)
          
          if @just_logged_in
            init_msg = [ 
              "USER blah blah blah :blah blah\n",
              "NICK #{@nick}\n",
              "JOIN #dradis\n" ]
            init_msg.each do |msg|
              @irc.write(msg, msg.size)
            end
            @just_logged_in = false
          end
        end
   
        def change_nick(new_nick)
          msg = "NICK #{new_nick}\r\n"
          @irc.write(msg, msg.size)
        end

        def displayError(socketError)
          case socketError
          when Qt::AbstractSocket::RemoteHostClosedError:
            ;
          when Qt::AbstractSocket::HostNotFoundError:
            Qt::MessageBox.information(self, tr("chat widget"),
              tr("The host was not found. Please check the " +
                  "host name and port settings."))
          when Qt::AbstractSocket::ConnectionRefusedError:
            Qt::MessageBox.information(self, tr("chat widget"),
              tr("The connection was refused by the peer. " +
                  "Make sure the server is running, " +
                  "and check that the host name and port " +
                  "settings are correct."))
          else
            Qt::MessageBox.information(self, tr("chat widget"),
              tr("The following error occurred: %s." %
                @irc.errorString))
          end   
        end        
        
        def send(msg)
          @irc.write(msg, msg.size)
        end
      end
      public
      class ChatWidget < Qt::Widget
        slots 'new_command()'
        
        def initialize
          super
          
          @bot = IrcBot.new()
          
          
          @layout1 = Qt::GridLayout.new(self)
          @splitter = Qt::Splitter.new(self)
          @browser = Qt::TextBrowser.new
          @browser.text= "your current <b>nick</b> is: #{@nick}. Use /nick to modify it."
          @users = Qt::ListWidget.new
          
          @splitter.addWidget(@browser)
          @splitter.addWidget(@users)
          
          @cmd = CommandLine.new
          @layout1.addWidget(@splitter, 0, 0, 1, 1)
          @layout1.addWidget(@cmd)
          
          connect @cmd, SIGNAL('returnPressed()'), self, SLOT('new_command()')
          
          @bot.browser = @browser
          @bot.users = @users
          @bot.exec
        end
        
        def new_command()
          input = @cmd.last_command
          return if ( (input == nil) || (input.size == 0) )
          
          if input =~ /^\/nick\s(.+)$/i
            @bot.change_nick($1)
          else
            @browser.append "#{@nick}> #{input}"
            msg = "PRIVMSG #dradis :#{input}\n"
            p msg
            @bot.send(msg)
            begin
              #output = @controller.parse_command(input)
            rescue
              output = "[error] #{$!}"
              $logger.error {$@.join("\n\t")}
              #@base.textConsole.append "error]\t" + $!
              #@base.textConsole.append $!.backtrace.sort.join("\n\t")
              #@base.textConsole.append $@.join("\n\t")
            end
          end
        end

      end
    end
  end
end

if $0 == __FILE__
  require 'commandline'
  a = Qt::Application.new(ARGV)
  w = Ui::Qt4::Widgets::ChatWidget.new
  w.show
  a.exec
end
