=begin
**
** notesbrowser.rb
** 05/SEP/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Qt custom widget that behaves as a standard command line. It keeps a 
** buffer of commands that can be accessed by pressing Up and Down keys.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'Qt4' if $0 == __FILE__
require '../widgets' if $0 == __FILE__


module Ui
  module Qt4
    module Widgets
      
      class NotesBrowser < Qt::Widget
        include NotesBrowserInterface
        
        slots 'invalidate!()', 'reload()'
        slots 'anchor_clicked(QUrl)'
        signals 'addNote(QString*, int)', 'editNote(QString*, int, int)'

        def initialize(parent=nil)
          super(parent)
          @layout = Qt::GridLayout.new(self)
          @t = Qt::TextBrowser.new
          @layout.addWidget(@t, 0, 0, 1, 1)
          @layout.margin = @layout.spacing = 0
          
          @html_default='<center><br/><img src="ui/qt/images/dradis.jpg"><h1>dradis 1.0</h1>etdsoft \'07</center>'
          @html_bar='<div><div>actions: <a href="add/#type#/#uid#">add note</a></div><div>show:#catlist# (<a href="filter/all">all</a>)</div></div>'
          @html_invalid='<center><br/><img src="ui/qt/images/dradis.jpg"><p>please, wait until the new revision is downloaded from the server...</p></center>'
          
          #@t = self
          @t.html= @last_html = @html_default
          connect @t, SIGNAL('anchorClicked(QUrl)'), self, SLOT('anchor_clicked(QUrl)')
          #self.openExternalLinks=  false
        end
      
        #----------------------------------------------------- private methods
        private
        
        def get_html_bar(type, uid, categories)
          cats =  ''
          categories.each do |cat|
            cats << " <a href=\"filter/#{cat}\">#{cat}</a>"
          end
          @html_bar.sub(/#type#/,type).sub(/#uid#/,uid.to_s).sub(/#catlist#/, cats)
        end
        
        def anchor_clicked(url)
          #return super if type.to_i == 2
          
          $logger.debug{"showing url: #{url.toString}"}
          #return if url.toString.size.zero?
          
          if url.toString == "#"
            @t.html = @last_html
          else
            params = url.toString.split('/')
            p params
            
            case params[0].intern
            when :filter
              filter_and_display(params[1])
            when :show
              @last_html = @t.toHtml
              @notes.each do |note|
                next unless note.uid == params[1].to_i
                extra = "[<a href=\"#\">back</a>]"
                @t.html = "<div><h3>#{note.category}</h3> <pre>#{note.text}</pre> <p>(<em>#{note.author}</em>) #{extra}[<a href=\"edit/#{note.uid}\">edit</a>]</p></div>"
              end
            when :add
              emit addNote(@type, @id)
            when :edit
              emit editNote(@type, @id, params[1].to_i)
            end
          end
          
        end

        def filter_and_display(filter='all')
          categories = {}
          
          @notes.each do |note|
            categories[note.category] = [] unless categories.has_key?(note.category)
            categories[note.category] << note
          end

          
          html = get_html_bar(@type, @id, categories.keys)
          if @notes.size == 0
            html += @html_default
          else
            categories.each do |name, notes|
              next unless ((filter=='all')||(filter==name))
              html << "<div><h3>#{name}</h3><ul>"
              notes.each do |n|
                extra = ''
                text = n.text
                if (text.size > 60)
                  text = text[0,59] << '...'
                  extra = "[<a href=\"show/#{n.uid}\">details</a>]"
                end
                html << "<li>#{text} (<em>#{n.author}</em>) #{extra}[<a href=\"edit/#{n.uid}\">edit</a>]</li>"
              end
              html << '</ul></div>'
            end
          end
          @t.setHtml( html )
        end
        #----------------------------------------------------- public methods
        public

	def invalidate!() @t.html = @html_invalid; end
        def reload() @t.reload; end
        def home() @t.html = @html_default; end
        
        def show_notes(annotatable_type, annotatable_id, note_list)
          @type = annotatable_type
          @id = annotatable_id
          @notes = note_list
          
          filter_and_display()
        end

        
      end # class NotesBrowser
      
    end # module Widgets
  end #module Qt4
end #module Ui

if $0 == __FILE__
  require 'logger'
  $logger = Logger.new(STDOUT)
  $logger.level = Logger::DEBUG

  a = Qt::Application.new(ARGV)
  w = Ui::Qt4::Widgets::NotesBrowser.new
  w.invalidate!('Service',1)
  w.show
  a.exec
end
