=begin
** window.rb
** 03/JUN/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Main Qt application window
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

#require custom widgets
require 'ui/qt/widgets'
require 'ui/qt/dialogs'

module Ui
  module Qt4
    # The main window of the Qt graphical interface. This class contains 
    # information about widgets, layouts, menus and icons used in the
    # main window.
    #
    # MainWindow will be used by DradisWindow to provide the different 
    # connections between the elements of the interface.
    class MainWindow
        
      attr_reader :centralwidget
      attr_reader :qgridLayout
      attr_reader :frame
      attr_reader :qvboxLayout
      attr_reader :textConsole
      attr_reader :commandLine
      #attr_reader :qhboxLayout
      attr_reader :splitter
      attr_reader :treeLeft
      #attr_reader :treeRight
      attr_reader :textRight
        
      attr_reader :menubar
      attr_reader :menu_File
      attr_reader :actionQuit
      attr_reader :menu_Connection
      attr_reader :menu_Help
      attr_reader :actionAbout
      attr_reader :statusbar
      
      attr_reader :trayIcon
    
      # Setup the menu bar, the menu items and the actions for each menu item.
      # Also setup the status bar for the main windo.
      def setupMenuAndStatus(mainWindow)
          
        #1: setup menu bar
        @menubar = Qt::MenuBar.new(mainWindow)
        @menubar.setObjectName("menubar")
        @menubar.setGeometry(Qt::Rect.new(0, 0, 621, 28))
          
        #1.1: setup menu items
        @menu_File = Qt::Menu.new(@menubar)
        @menu_File.setObjectName("menu_File")
        @menu_Connection = Qt::Menu.new(@menubar)
        @menu_Connection.setObjectName("menu_Connection")
        @menu_Help = Qt::Menu.new(@menubar)
        @menu_Help.setObjectName("menu_Help")


        #1.2: add actions to the items
        #1.2.1: create the actions
        #------------------------------------- File
        @actionQuit = Qt::Action.new(mainWindow)
        @actionQuit.setObjectName("actionQuit")
        @actionQuit.shortcut = Qt::KeySequence.new( 'Ctrl+Q' )
        @actionQuit.setIcon(mainWindow.style().standardIcon(Qt::Style::SP_TitleBarCloseButton))
        #------------------------------------- Window
        #------------------------------------- Help
        @actionAbout = Qt::Action.new(mainWindow)
        @actionAbout.setObjectName("actionAbout")

        #1.2.2: add to the items
        @menu_File.addAction(@actionQuit)
        @menu_Help.addAction(@actionAbout)
          
        #1.3: add the items to the bar
        @menubar.addAction(@menu_File.menuAction())
        @menubar.addAction(@menu_Connection.menuAction())
        @menubar.addAction(@menu_Help.menuAction())
          
          
        #1.4: add the bar to the main window
        mainWindow.setMenuBar(@menubar)

          
          
        #2: setup status bar
        @statusbar = Qt::StatusBar.new(mainWindow)
        @statusbar.setObjectName("statusbar")
        mainWindow.setStatusBar(@statusbar)
      end
      
      # Setup the system tray icon and menu
      def setupSysTray(mainWindow)
        
        if !Qt::SystemTrayIcon.isSystemTrayAvailable
          Qt::MessageBox.critical(nil, Qt::Object.tr("Systray"),
          Qt::Object.tr("I couldn't detect any system tray " \
          "on this system."))
        else
          #          @trayIconMenu = Qt::Menu.new(self) do |t|
          #            t.addAction(@minimizeAction)
          #            t.addAction(@maximizeAction)
          #            t.addAction(@restoreAction)
          #            t.addSeparator()
          #            t.addAction(@quitAction)
          #          end
          icon = Qt::Icon.new("ui/qt/images/bad.svg")
          @trayIcon = Qt::SystemTrayIcon.new(mainWindow)
          @trayIcon.contextMenu = @menu_File
          @trayIcon.icon = icon
          @trayIcon.show
          @trayIcon.toolTip = 'dradis'
          mainWindow.setWindowIcon(icon)
        end       
      end

      # i18n the strings of the main window
      def retranslateUi(mainWindow)
        mainWindow.setWindowTitle(Qt::Application.translate("MainWindow", "MainWindow", nil, Qt::Application::UnicodeUTF8))
          
        @actionQuit.setText(Qt::Application.translate("MainWindow", "&Quit", nil, Qt::Application::UnicodeUTF8))
        @actionQuit.statusTip = Qt::Application.translate("MainWindow", "quit the application", nil, Qt::Application::UnicodeUTF8)

        @actionAbout.setText(Qt::Application.translate("MainWindow", "&About", nil, Qt::Application::UnicodeUTF8))
        @actionAbout.statusTip = Qt::Application.translate("MainWindow", "information about the applications", nil, Qt::Application::UnicodeUTF8)

        @menu_File.setTitle(Qt::Application.translate("MainWindow", "&File", nil, Qt::Application::UnicodeUTF8))
        @menu_Connection.setTitle(Qt::Application.translate("MainWindow", "&Connection", nil, Qt::Application::UnicodeUTF8))
        @menu_Help.setTitle(Qt::Application.translate("MainWindow", "&Help", nil, Qt::Application::UnicodeUTF8))
          
      end # retranslateUi

      # Setup the different widgets and layouts of the main window.
      def setupUi(mainWindow)
        mainWindow.setObjectName("mainWindow")
        $qApp.setFont(Qt::Font.new("Verdana", 11));
        
        
        @centralwidget = Qt::Widget.new(mainWindow)
        @centralwidget.setObjectName("centralwidget")
        @qgridLayout = Qt::GridLayout.new(@centralwidget)
        @qgridLayout.spacing = 6
        @qgridLayout.margin = 9
        @qgridLayout.setObjectName("qgridLayout")
        
        dock = Qt::DockWidget.new(mainWindow)
        dock_features = Qt::DockWidget::DockWidgetMovable | Qt::DockWidget::DockWidgetFloatable
        dock.features = dock_features.to_i
        dock.windowTitle = 'console'
        
        #docking chat
        chat = Qt::DockWidget.new(mainWindow)
        #qt 4.2 dock.setFeatures(Qt::DockWidget::DockWidgetMovable | Qt::DockWidget::DockWidgetFloatable)
        chat_features = Qt::DockWidget::DockWidgetMovable | Qt::DockWidget::DockWidgetFloatable
        chat.features = chat_features.to_i
        chat.windowTitle = 'chat'
        
        
        @frame = Qt::Frame.new(@centralwidget)
        @frame.setObjectName("frame")
        @frame.setFrameShape(Qt::Frame::StyledPanel)
        @frame.setFrameShadow(Qt::Frame::Raised)
        
        @consoleLayout = Qt::GridLayout.new(@frame)
        @consoleLayout.spacing = 6
        @consoleLayout.margin = 9
        @consoleLayout.setObjectName("consoleLayout")
        
        @textConsole = Qt::TextBrowser.new(@frame)
        @textConsole.setObjectName("textConsole")
        @consoleLayout.addWidget(@textConsole, 0, 0, 1, 1)
    
        @commandLine = Ui::Qt4::Widgets::CommandLine.new(@frame)
        @commandLine.setObjectName("commandLine")
    
        @consoleLayout.addWidget(@commandLine, 1, 0, 1, 1)


        dock.widget = @frame
        mainWindow.addDockWidget(Qt::BottomDockWidgetArea, dock)
        chat.widget = Ui::Qt4::Widgets::ChatWidget.new
        mainWindow.addDockWidget(Qt::BottomDockWidgetArea, chat)

    
        @splitter = Qt::Splitter.new(@centralwidget)
        @splitter.setObjectName("splitter")
        @splitter.setOrientation(Qt::Horizontal)
    
        @treeLeft = Qt::TreeWidget.new(@centralwidget)
        @treeLeft.setObjectName("treeLeft")
        @treeLeft.alternatingRowColors = true
    
        @splitter.addWidget(@treeLeft)
    
        @textRight = Ui::Qt4::Widgets::NotesBrowser.new(@centralwidget)
        @textRight.setObjectName("textRight")
        @splitter.addWidget(@textRight)
    
        @qgridLayout.addWidget(@splitter, 0, 0, 1, 1)
    
        mainWindow.setCentralWidget(@centralwidget)
      


        Qt::Widget::setTabOrder(@commandLine, @textConsole)
        Qt::Widget::setTabOrder(@textConsole, @treeLeft)
        Qt::Widget::setTabOrder(@treeLeft, @treeRight)

        setupMenuAndStatus(mainWindow)
        setupSysTray(mainWindow)
        retranslateUi(mainWindow)
    
        size = Qt::Size.new(621, 600)
        size = size.expandedTo(mainWindow.minimumSizeHint())
        mainWindow.resize(size)
    
    
        Qt::MetaObject.connectSlotsByName(mainWindow)
      end # setupUi
    

    end
    
    # DradisWindow creates an interface window on top of MainWindow Qt elements.
    # DradisWindow defines connections between the elements of the interface.
    # It also defines a set of operations connected to the input and actions
    # performed by the user.
    #
    # MainWindow can be seen as the structure of the building while DradisWindow
    # provides the funcionality to the different _rooms_ and _levels_.
    class DradisWindow < Qt::MainWindow
      slots 'console_new_command()', 'timeout_updates()', 'item_clicked(QTreeWidgetItem*,int)', 'add_note(QString*,int)', 'edit_note(QString*,int,int)', 'iconActivated(QSystemTrayIcon::ActivationReason)'
      attr_writer :controller
  
      #===================================================== private methods
      private

      # Every time the clock ticks, we check for an update in the remote 
      # information repository. The clock period can be adjusted in the
      # config file.
      def timeout_updates
        old_revision = @controller.model.revision
        out = @controller.refresh_model()
        @base.textConsole.append( out ) unless (out.nil?)
        if (old_revision != @controller.model.revision)
          # update the view
          update_view
          #@base.textConsole.append 'view should be updated'
        end
      end

      # When the user hits *Enter* in the command line, this method is called.
      # the method will delegate the parsing and execution of the command to
      # the controller. See ControllerInterface#parse_command
      def console_new_command
        #Qt::MessageBox.information self, 'App',  @base.commandLine.text
        input = @base.commandLine.last_command
        return if ( (input == nil) || (input.size == 0) )
    
        @base.textConsole.append "dradis> #{input}"
        
        #@last_commands << input
        begin
          output = @controller.parse_command(input)
        rescue
          output = "[error] #{$!}"
          $logger.error {$@.join("\n\t")}
          #@base.textConsole.append "error]\t" + $!
          #@base.textConsole.append $!.backtrace.sort.join("\n\t")
          #@base.textConsole.append $@.join("\n\t")
        end
        @base.textConsole.append(output.to_s)
        @base.commandLine.clear
      end
    
      # When the user clicks on one of the elements of the lefthand side tree,
      # this method is called to show the notes of the clicked element in the
      # righthand side of the window.
      def item_clicked(tree,column)
        annotatable = nil
        type = nil
        if (tree.parent == nil)
          @controller.model.hosts.each do |host|
            next unless host.address == tree.text(column).split()[1]
            annotatable = host
            type = 'Host'
          end
        else
          @controller.model.hosts.each do |host|
            next unless host.address == tree.parent.text(column).split()[1]
            $logger.debug{ "parent identified: #{host.address}"}
            banner = tree.text(column).split()
            banner.shift
            host.services.each do |srv|
              next unless "#{srv.name} (#{srv.protocol}/#{srv.port})" == banner.join(' ')
              annotatable = srv
              type = 'Service'
            end
          end
        end
        
        @base.textRight.show_notes(type, annotatable.uid, annotatable.notes)
        return
        
      end
      
      # This method is called when new information has arrived from the server.
      # The method populates the lefthand side tree with Hosts and Services.
      def update_view
        @base.treeLeft.clear
        @controller.model.hosts.each do |host|
          item = Qt::TreeWidgetItem.new()
          item.setText(0, "#{host['uid']}: #{host['address']}")
          @base.treeLeft.addTopLevelItem(item)
          parent = @base.treeLeft.topLevelItem(@base.treeLeft.topLevelItemCount-1)
          host.services.each do |srv|
            item2 = Qt::TreeWidgetItem.new()
            item2.setText(0, "#{srv.uid}: #{srv.name} (#{srv.protocol}/#{srv.port})")
            parent.insertChild(parent.childCount, item2)
          end
          @base.treeLeft.setItemExpanded(parent,true) if host.services.size > 0
        end
        @base.textRight.home
      end

  
      
      # Displays the popup dialog to add a new note.
      def add_note(annotatable_type, annotatable_id)
        d = Ui::Qt4::Dialogs::NotesDialog.new
        d.categories = @controller.model.categories
        d.exec

        if d.accepted
          @controller.add_note(annotatable_type, annotatable_id, d.author, d.category, d.text)
          @base.textRight.invalidate!
        else
          @base.textRight.reload
        end
      end

      # Displays the popup dialog to edit an existing note.
      def edit_note(annotatable_type, annotatable_id, note_id)
            
        note = @controller.find_note(note_id)
            
        d = Ui::Qt4::Dialogs::NotesDialog.new
        d.categories = @controller.model.categories
        d.edit_note(note)
        d.exec

        if d.accepted 
          if (( d.author != note.author ) |
            ( d.category != note.category ) |
            ( d.text != note.text )
            )
            @controller.edit_note(note_id, d.author, d.category, d.text)
            @base.textRight.invalidate!
          else
            @base.textRight.reload
          end
        else
          @base.textRight.reload
        end
      end
      
      # When the user clicks in the system tray icon (if available) this method
      # handles the event.
      def iconActivated(reason)
        case reason
        when Qt::SystemTrayIcon::Trigger:
        when Qt::SystemTrayIcon::DoubleClick:
          self.show
        when Qt::SystemTrayIcon::MiddleClick:
          @base.trayIcon.showMessage('dradis says:', 'all your base are belong to us!')#, Qt::Icon.new("ui/qt/images/heart.svg"), 3 * 1000)
        else
            
        end
      end
      
      #===================================================== public methods
      public
      # Create the window, instantiate the widgets, connect signals and slots.
      # Start the timer that will query the server for updates. The refresh
      # period is defined in the config file.
      def initialize(parent=nil)
        super #call Qt::Widget constructor
        @base=Ui::Qt4::MainWindow.new
        @base.setupUi self #this is where the widget construction happens
    
        @timer_updates = Qt::Timer.new( self )
        connect( @timer_updates, SIGNAL( 'timeout()' ), SLOT( 'timeout_updates()' ) )
    
        connect @base.actionQuit, SIGNAL('triggered()'), $qApp, SLOT('quit()')
        connect @base.commandLine, SIGNAL('returnPressed()'), self, SLOT('console_new_command()')
        
        connect @base.treeLeft, SIGNAL('itemClicked(QTreeWidgetItem*,int)'), self, SLOT('item_clicked(QTreeWidgetItem*,int)')
        connect @base.textRight, SIGNAL('addNote(QString*, int)'), self, SLOT('add_note(QString*,int)')
        connect @base.textRight, SIGNAL('editNote(QString*, int, int)'), self, SLOT('edit_note(QString*,int,int)')

        if Qt::SystemTrayIcon.isSystemTrayAvailable
          connect(@base.trayIcon, SIGNAL('activated(QSystemTrayIcon::ActivationReason)'),
          self, SLOT('iconActivated(QSystemTrayIcon::ActivationReason)'))
        end       
    
        @timer_updates.start( $config.get_option(:ui_refresh).to_i )
        #@updateontick = false
    
        #TODO: @base.treeLeft.installEventFilter(self)
        #@base.commandLine.installEventFilter(self)
    
        @base.treeLeft.setHeaderLabels(['hosts'])
        @par = true
        @last_commands = []
        #@revision = 0
        
      end
      

    end
    
  end
  
end
