=begin
**
** wx.rb
** 01/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  This script module contains all the User Interface related code using the 
** wxWidgets implementation
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

# first check that the gem is installed, if it is not, provide some helpful
# clue
begin
  require 'wx'
  if Wx::WXRUBY_VERSION != '1.9.9'
    msg = "The version of the wxruby library (#{Wx::WXRUBY_VERSION}) is not " +
      'supported. Please install version 1.9.9 using: gem install wxruby -v=1.9.9'
    $logger.error(msg)
    raise LoadError.new(msg)
  end
rescue LoadError => no_wx_err
  begin
    require 'rubygems'
    gem 'wxruby', '=1.9.9'
    require 'wx'
  rescue LoadError => no_wx_err
    $logger.debug no_wx_err.message
    $logger.error('wxruby (1.9.9) gem not found. install it using: gem install wxruby -v=1.9.9')
    raise no_wx_err
  end
end

require 'ui/dialogs'
require 'ui/widgets'
require 'ui/wx/window'

# Different user interfaces will be grouped under this module.
module Ui
  # Graphical user interface classes and modules. Based on the wxWidgets
  # framework (http://www.wxwidgets.org/). In order for this to work you will 
  # need the wxruby gem (http://wxruby.rubyforge.org/) installed.
  module WxWidgets
    # Main Wx Application, handles the creation of main window, window title, etc.
    class DradisGui < Wx::App
      include Core::Interfaces::View
      
      def on_init
        @frame = Ui::WxWidgets::DradisWindow.new( nil, 
                           Wx::ID_ANY, 
                           "dradis",
                           Wx::DEFAULT_POSITION,
                           Wx::Size.new(800, 650) 
                        )
        set_top_window(@frame)
        @frame.controller = @controller
        @frame.show
        return true                       
      end
      def setup(new_controller)
        @controller = new_controller
        # register the view as an observer of the exceptions thrown by the 
        # controller
        @controller.add_observer(:exception, self)
      end
      def run()      
        main_loop
      end
      def teardown()
        @frame.teardown
      end
      
      def notify(event, *args)
        if @frame.nil?
          # an error accured during initialisation
          $logger.error{ event }
          $logger.error{ args }
        else
          @frame.notify(event, *args)
        end
      end
    end
  end
end
