=begin
**
** console.rb
** 03/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  wxWidgets implementation of the ConsoleInterface
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Ui
  module WxWidgets
    module Widgets
      
      class ConsoleNewCommandEvent < Wx::CommandEvent
        # Create a new unique constant identifier, associate this class
        # with events of that identifier, and create a shortcut 'evt_new_command'
        # method for setting up this handler.
        EVT_NEW_COMMAND = Wx::EvtHandler.register_class(self, nil, 'evt_new_command', 1)

        def initialize(console, line)
          # The constant id is the arg to super
          super(EVT_NEW_COMMAND)
          # client_data should be used to store any information associated
          # with the event.
          self.client_data = { :line => line  }
          self.id = console.get_id
        end

        def line
          client_data[:line]
        end

      end
      
      class BufferedTextCtrl < Wx::TextCtrl
        def initialize(parent, name)
          super(
            parent, 
            Wx::ID_ANY,
            '',
            Wx::DEFAULT_POSITION, 
            Wx::DEFAULT_SIZE, 
            Wx::TE_PROCESS_ENTER, 
            Wx::DEFAULT_VALIDATOR,
            name)

          evt_char do | event | on_char(event) end
          
          #initialize internal history buffer
          @history = []
          @pointer = 0
          @up = true             
        end
        
        def on_char(event)
          case (event.key_code)
          when Wx::K_UP
            if ((@history.size > 0) && (@pointer >= 0) )
              if (@pointer == @history.size)
                @history << self.get_value
              end
              @pointer -= 1 if @pointer > 0
              self.set_value( @history[@pointer] )
              self.set_insertion_point_end()
            end            
          when Wx::K_DOWN
            if ((@history.size > 0) && (@pointer < @history.size) )
              @pointer += 1 if @pointer < @history.size - 1
              self.set_value( @history[@pointer] )
              self.set_insertion_point_end()
            end            
          when Wx::K_RETURN
            $logger.debug(@history)
            #keep an eye on the last entry to avoid empty entries in the list
            if ((@history.size > 0) && (@history.last.strip.size == 0) )
              @history.pop
            end

            line = self.get_value.strip

            if line.size > 0
              @history << line
              @pointer = @history.size
              self.clear
            end            
            $logger.debug(@history)
          end
          event.skip
        end
        # Returns the last command issued by the user. This method should be 
        # used when the parent window captures a +returnPressed+ event to
        # examine the las line inputted by the user.
        def last_command()
          @history.last
        end        
      end
      class Console < Wx::Panel
        include ConsoleInterface
        #===================================================== private methods
        private
        
        def on_text_enter
          line = @input.last_command
          append("dradis> #{line}")
          event_handler.process_event( ConsoleNewCommandEvent.new(self, line) )
        end
        
        #===================================================== public methods
        public    

        def initialize(parent=nil)
          super(parent, Wx::ID_ANY, Wx::DEFAULT_POSITION, Wx::Size.new(400, 300) )
          
          @output = Wx::TextCtrl.new(
            self,  
            Wx::ID_ANY,
            "dradis> Welcome to dradis\n",
            Wx::DEFAULT_POSITION, 
            Wx::DEFAULT_SIZE, 
            Wx::TE_MULTILINE|Wx::TE_READONLY, 
            Wx::DEFAULT_VALIDATOR,
            'output')
          
          @input = BufferedTextCtrl.new(self, 'input')
          
          
          box = Wx::BoxSizer.new(Wx::VERTICAL)
          box.add( @output, 1, Wx::EXPAND )
          box.add( @input, 0, Wx::EXPAND )
          set_sizer(box)
          
          evt_text_enter(@input.get_id) { | event | on_text_enter }       
        end
        
        def append(line)
          @output.append_text("#{line}\n")
        end
      end
    end
  end
end