=begin
**
** hoststree.rb
** 03/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  wxWidgets implementation of the HostsBrowserInterface
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Ui
  module WxWidgets
    module Widgets
      class NodesBrowserEvent < Wx::CommandEvent
        # Create a new unique constant identifier, associate this class
        # with events of that identifier, and create a shortcut 'evt_rename_node'
        # and 'evt_reassign_node'.
        EVT_RENAME_NODE = Wx::EvtHandler.register_class(self, nil, 'evt_rename_node', 1)
        EVT_REASSIGN_NODE = Wx::EvtHandler.register_class(self, nil, 'evt_reassign_node', 1)
        # TODO: is this required?
        EVT_ADD_CHILD_NODE = Wx::EvtHandler.register_class(self, nil, 'evt_add_child_node', 1)
        EVT_CLICK_NODE = Wx::EvtHandler.register_class(self, nil, 'evt_click_node', 1)
        EVT_RIGHT_CLICK_NODE = Wx::EvtHandler.register_class(self, nil, 'evt_right_click_node', 1)
        
        def initialize(nodes_browser, event_id, node)
          super(event_id)
          self.client_data = node
          self.id = nodes_browser.get_id
        end
        
        def node
          self.client_data
        end
      end
              
      class HostsBrowser < Wx::TreeCtrl
        include HostsBrowserInterface
        
        #===================================================== private methods
        private
        consts = %w[
          ID_NodesTreeAddChild
          ID_NodesTreeProperties
          ]
        consts.each_with_index do | c, i |
          const_set(c, 3000 + i) 
        end        
        
        def add_node(parent,node)
          #this_node = self.append_item(parent, "#{node.id}: #{node.label} | #{node.parent_id} | #{node.type_id}", 0)
          this_node = self.append_item(parent, "#{node.id}: #{node.label}", 0)
          self.set_item_data(this_node, node)

          @model.node_children(node.id).each do |child| add_node(this_node, child) end
        end
        def setup_dummy_content
           root = self.add_root("Wx::AUI Project", 0)
           items = []
           1.upto(5) { | i | items << self.append_item(root, "Item #{i}", 0) }
   
           items.each do | id |
             1.upto(5) { | i | self.append_item(id, "Subitem #{i}", 0) }
           end
   
           self.expand(root)          
        end          

        #----------------------------------------------------------------------
        #----------------------------------------------------- event handlers
        #----------------------------------------------------------------------
        def on_begin_label_edit(event)
          label = self.get_item_text(event.get_item)
          $logger.debug{ "on_begin_label_edit| #{event.get_item}. #{label}" }
          @node_id = label.scan(/(\d*):/).first.first.to_i
          @node_label = label.scan(/\s(.*)/).first.first
          
          self.set_item_text(event.get_item, @node_label)
          puts "label: #{@node_label}"
          puts "item: #{@node_id}"
        end
        
        def on_end_label_edit(event)
          $logger.debug{ "on_end_label_edit| #{event.get_item}." }
          
          new_label = ''
          # if the label wasn't modified
          if event.is_edit_cancelled()
            new_label = @node_label
          else
            new_label = event.get_label
          end
          puts "item_text: #{self.get_item_text(event.get_item)}"
          self.set_item_text(event.get_item, "#{@node_id}: #{new_label}")
          node_data = self.get_item_data(event.get_item)
          node_data.label = new_label
          self.set_item_data(event.get_item, node_data)
          puts "label: #{event.get_label}"
          puts "item_text: #{self.get_item_text(event.get_item)}"

          # bubble an event to the parent window
          unless event.is_edit_cancelled()
            event_handler.process_event( NodesBrowserEvent.new(self, NodesBrowserEvent::EVT_RENAME_NODE, node_data))
          end
          event.veto()
        end
        
        def on_begin_drag(event)   
          $logger.debug { 'on_begin_drag'}
          event.allow
          @node_drag = event.get_item
          
          # in order to prevent a parent node to be dropped into one of it's
          # children we create an array with all the TreeItemIds that are 
          # forbidden for the current dragged node.
          # note that if a node is dropped over itself annihilation will happen :)
          @forbidden_drops = []
          
          self.traverse(@node_drag) do |child|
            @forbidden_drops << child
          end
          p @forbidden_drops
        end
     
        def rearrange_node(node_id, new_parent_id)        
          $logger.debug { "rearranging node [#{self.get_item_text(node_id)}]"}
          
          # 1: create a new node under the parent
          node_data = self.get_item_data(node_id)
          node_data.parent_id = self.get_item_data(new_parent_id).id
          $logger.debug { node_data.inspect }
          new_id = self.append_item(
                    new_parent_id, 
                    self.get_item_text(node_id),
                    self.get_item_image(node_id, Wx::TREE_ITEM_ICON_NORMAL),
                    self.get_item_image(node_id, Wx::TREE_ITEM_ICON_SELECTED),
                    node_data
                    )
          
          # 2: rearrange the children
          self.get_children(node_id).each do |child|
            rearrange_node(child, new_id)
          end
          
          # 3: delete the original node
          self.delete(node_id)     
          return new_id      
        end

        def on_end_drag(event)
          return if event.get_item.zero?
            
          
          @node_drop = event.get_item
          $logger.debug { "on_end_drag | dropping #{self.get_item_text(@node_drag)} over #{self.get_item_text(@node_drop)}" }
          
          # prevent a node to be droped into as a child of itself!
          return if @forbidden_drops.include?(@node_drop)
          
          # re-arrange the nodes. not only the dropped node, but also the it's 
          # children
          # TODO: maybe there is no need for this and the reassignment will be
          # done at a database level.
          new_node_id = rearrange_node(@node_drag, @node_drop)
          
          # bubble the event to the parent window
          event_handler.process_event( 
            NodesBrowserEvent.new(
              self, 
              NodesBrowserEvent::EVT_REASSIGN_NODE, 
              self.get_item_data(new_node_id)
            )
          )
          event.veto
        end
        
        def on_tree_clicked(event)
          item_id = event.get_item
          return if item_id.zero?
          
          # although with the root item event.get_item should return 0, this is
          # not always the case. TODO: investigate!
          return if (self.get_item_text(item_id) == 'dradis')

          $logger.debug { "on_tree_clicked| node #{item_id} has been clicked. [#{self.get_item_text(item_id)}]" }
          
          unless @selected_node.nil?
            self.set_item_bold(@selected_node, false)
          end
          
          self.set_item_bold(item_id)
          @selected_node = item_id

          event_handler.process_event(NodesBrowserEvent.new(self, NodesBrowserEvent::EVT_CLICK_NODE, self.get_item_data(item_id)))
        end   
             
        def on_tree_item_right_click(event)
          $logger.debug { "on_tree_item_right_click| #{event.get_item}: #{self.get_item_text(@selected_node)}" }
          #self.popup_menu(@popup_menu)
          event_handler.process_event( 
            NodesBrowserEvent.new(
              self, 
              NodesBrowserEvent::EVT_RIGHT_CLICK_NODE, 
              self.get_item_data(event.get_item)
            )
          )          
        end      
        
        #----------------------------------------------------- popup menu handlers
        # TODO: these are not needed any more. Now the popup menu is handled by 
        # the parent window
        def on_add_child(event)
          node = self.get_item_data(@selected_node)
          dialog = Wx::TextEntryDialog.new(self, 
                          'Please provide a label for the new node:', 
                          'dradis 2.0 info message', 
                          "child of #{node.label}")

          if (dialog.show_modal() == Wx::ID_OK)         
            node.parent_id = node.id
            node.label = dialog.get_value
            event_handler.process_event(NodesBrowserEvent.new(self, NodesBrowserEvent::EVT_ADD_CHILD_NODE, node))
          end
          
        end

        def on_item_properties(event)
          Wx::MessageDialog.new(self, 
              "sorry, but I'm afraid this is unimplemented at the moment",
              'dradis 2.0 info message', Wx::OK | Wx::ICON_INFORMATION).show_modal
        end
        
        #----------------------------------------------------------------------
        #----------------------------------------------------- setup the interface
        #----------------------------------------------------------------------
        def setup_menus
          # popup menu for the nodes tree
          @popup_menu = Wx::Menu.new
          @popup_menu.append(ID_NodesTreeAddChild, 'add child')
          @popup_menu.append_separator()
          @popup_menu.append(ID_NodesTreeProperties, 'properties')
        end
        
        def setup_images
          img_list = Wx::ImageList.new(16, 16, true, 2)
          img_list.add( 
            Wx::ArtProvider::get_bitmap( 
              Wx::ART_FOLDER,
              Wx::ART_OTHER,
              Wx::Size.new(16, 16) 
            ) 
          )
          img_list.add( 
            Wx::ArtProvider::get_bitmap( 
              Wx::ART_NORMAL_FILE,
              Wx::ART_OTHER,
              Wx::Size.new(16, 16) 
            ) 
          )
          
          self.set_image_list(img_list)          
        end
        
        def setup_events
          # left and right click
          evt_tree_sel_changed(self.get_id) { |e| on_tree_clicked(e) }
          evt_tree_item_right_click(self.get_id) { |e| on_tree_item_right_click(e) }

          # popup menus
          evt_menu(ID_NodesTreeAddChild) { |e| on_add_child(e) }
          evt_menu(ID_NodesTreeProperties) { |e| on_item_properties(e) }
          
          
          # changes in the labels of the nodes
          evt_tree_begin_label_edit(self.get_id) { |event| on_begin_label_edit(event) }
          evt_tree_end_label_edit(self.get_id) { |event| on_end_label_edit(event) }
          
          # drag'n'drop
          evt_tree_begin_drag(self.get_id) { |event| on_begin_drag(event) }
          evt_tree_end_drag(self.get_id) { |event| on_end_drag(event) }
        end
      
      
        #===================================================== public methods
        public
        
        def initialize(parent=nil)
          super(parent, 
            Wx::ID_ANY, 
            Wx::DEFAULT_POSITION,
            Wx::Size.new(200, 250),
            Wx::TR_DEFAULT_STYLE|Wx::NO_BORDER|Wx::TR_HIDE_ROOT|Wx::TR_EDIT_LABELS)
          
          setup_menus
          setup_images
          setup_events
        
          # setup_dummy_content
          self.disable
          
          # initialize instance vars
          @old_label = nil
        end
        
        def update_view(model)
          self.disable
          self.delete_all_items
          @selected_node = nil
          
          @model = model
          root = self.add_root('dradis', 0)
          # find the root nodes (parent_id == nil)
          @model.node_children(nil).each do |node|
            add_node(root, node)
          end
          self.expand_all
          self.enable
        end
        
      end
      
    end
    
  end
  
end