=begin
**
** notesbrowser.rb
** 03/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  wxWidgets implementation of the NotesBrowserInterface.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end


module Ui
  module WxWidgets
    module Widgets
      class NotesBrowserEvent < Wx::CommandEvent
        # Create a new unique constant identifier, associate this class
        # with events of that identifier, and create a shortcut 'evt_new_command'
        # method for setting up this handler.
        EVT_ADD_NOTE = Wx::EvtHandler.register_class(self, nil, 'evt_add_note', 1)
        EVT_EDIT_NOTE = Wx::EvtHandler.register_class(self, nil, 'evt_edit_note', 1)

        def initialize(notes_browser, event_id, node_id, note_id = nil)
          # The constant id is the arg to super
          super(event_id)
          # client_data should be used to store any information associated
          # with the event.
          self.client_data = { 
            :node_id => node_id,
            :note_id => note_id
            }
          self.id = notes_browser.get_id
        end
               
        def node_id
          client_data[:node_id]
        end          
        def note_id
          client_data.fetch(:note_id, nil)
        end          

      end

      class NotesBrowser < Wx::HtmlWindow
        attr_reader :html_src
        
        include NotesBrowserInterface
        
        #===================================================== private methods
        private
        
        def get_html_bar(node_id, categories)
          cats =  ''
          categories.each do |cat|
            cats << " <a href=\"filter/#{cat}\">#{cat}</a>"
          end
          HTML[:bar].sub(/#node_id#/,node_id.to_s).sub(/#catlist#/, cats)
        end
        def filter_and_display(filter='all')
          categories = {}
          
          @notes.each do |note|
            cat = @model.category_find(note.category_id)
            categories[cat.name] = [] unless categories.has_key?(cat.name)
            categories[cat.name] << note
          end

          
          html = get_html_bar(@node_id, categories.keys)
          if @notes.size == 0
            html += HTML[:home]
          else
            categories.each do |name, notes|
              next unless ((filter=='all')||(filter==name))
              html << "<div><h3>#{name}</h3><ul>"
              notes.each do |n|
                extra = ''
                text = n.text
                if (text.size > 60)
                  text = text[0,59] << '...'
                  extra = "[<a href=\"show/#{n.id}\">details</a>]"
                end
                html << "<li>#{CGI::escapeHTML(text)} (<em>#{n.author}</em>) #{extra}[<a href=\"edit/#{n.id}\">edit</a>]</li>"
              end
              html << '</ul></div>'
            end
          end
          self.set_page( html )
          @html_src = html
        end
        
        
        def on_link_clicked(event)
          url = event.get_link_info.get_href 
          $logger.debug{"showing url: #{url}"}

          if url == '#'
            self.set_page( @last_html )
          else
            params = url.split('/')
            
            case params[0].intern
            when :filter
              filter_and_display(params[1])
            when :show
              @last_html = @html_src
              @notes.each do |note|
                next unless note.id == params[1].to_i
                extra = "[<a href=\"#\">back</a>]"
                self.set_page( "<div><h3>#{CGI::escapeHTML(@model.category_find(note.category_id).name)}</h3> <pre>#{CGI::escapeHTML(note.text)}</pre> <p>(<em>#{CGI::escapeHTML(note.author)}</em>) #{extra}[<a href=\"edit/#{note.id}\">edit</a>]</p></div>" )
              end
            when :add
              event_handler.process_event( NotesBrowserEvent.new(self, NotesBrowserEvent::EVT_ADD_NOTE, @node_id) )
            when :edit
              event_handler.process_event( NotesBrowserEvent.new(self, NotesBrowserEvent::EVT_EDIT_NOTE, @node_id, params[1].to_i) )
            end
          end          
        end
        #===================================================== public methods
        public         
        
        def initialize(parent=nil)
          super(parent, Wx::ID_ANY, Wx::DEFAULT_POSITION, Wx::Size.new(400, 300) )
          
          @model = []
          
          evt_html_link_clicked(self.get_id) { |event| on_link_clicked(event)}
          home
        end

        def set_page(html)
          super html
          @html_src = html
        end
        
        def home()
          self.set_page(HTML[:home])
        end
        
        def show_notes(node_id, note_list)
          @node_id = node_id
          @notes = note_list
          
          filter_and_display()
        end        
        
        def invalidate!() self.set_page(HTML[:invalid]); end
        
        def update_view(model)
          @model = model
          home
        end
      end
    end
  end
end
