=begin
**
** propertiespane.rb
** 29/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  wxWidgets implementation of the PropertiesPaneInterface.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Ui
  module WxWidgets
    module Widgets
      
      class SingleColListCtrl < Wx::ListCtrl
        #===================================================== private methods
        private
        
        def onSize(event)
         event.skip()
         set_column_width(0, get_client_size.get_width - 20)
        end

        #===================================================== public methods
        public

        def initialize(parent, multiple, options, value)
         flags = Wx::LC_REPORT|Wx::LC_NO_HEADER
         if not multiple
           flags |= Wx::LC_SINGLE_SEL
         end
         super(parent, 
                -1,
                Wx::DEFAULT_POSITION, 
                Wx::Size.new(100, 100),
                flags)
         evt_size() {|event| onSize(event) }

         insert_column(0, 'Values')
         @options = options
         @options.each do |option|
           insert_item(get_item_count, option)
           if value.include?(option)
             set_item_state(get_item_count - 1, Wx::LIST_STATE_SELECTED, Wx::LIST_STATE_SELECTED)
           end
         end
         scroll_list(0, 0)
        end

        def get_selections
          out = []
          item = -1
          while(true)
            item = get_next_item(item, Wx::LIST_NEXT_ALL, Wx::LIST_STATE_SELECTED)  
            break if item == -1
            out << get_item_text(item)
          end
          out
        end
        
        def get_options
          @options
        end
      end
      
      class PropertiesPane < Wx::Panel
        include Ui::PropertiesPaneInterface
        EVT_PROPERTIES_APPLY = Wx::EvtHandler.register_class(Wx::CommandEvent, nil, 'evt_properties_apply', 1)
        
        #===================================================== private methods
        private
        
        def home
          @grid.set_cols(1)
          @grid.add_growable_col( 0 )
          @grid.add(  
            Wx::StaticText.new( self, Wx::ID_ANY, MSG[:home] ),
            0, 
            Wx::ALL|Wx::ALIGN_CENTRE, 
            5
          )
          @grid.layout
        end
        
        #----------------------------------------------------------------------
        #----------------------------------------------------- event handlers
        #----------------------------------------------------------------------
        
        def on_value_changed(event, original_value)
          return if @skip_apply
          return if @apply.is_enabled # if some widged enabled the button, skip
          @apply.enable
        end
        
        #----------------------------------------------------------------------
        #----------------------------------------------------- setup the interface
        #----------------------------------------------------------------------
        
        
        def clear_grid()
          # if it is the first time we are displaying something, remove 
          # the growable column set in home() and prepare the grid to
          # display two columns
          if (@grid.get_children.size == 1)
            @grid.remove_growable_col( 0 )
            @grid.add_growable_col( 1 )
            @grid.set_cols(2)
          end

          while(@grid.show(0, false))
            item = @grid.get_item(0)
            if (item.is_window)
              disconnect( @grid.get_item(0).get_window.get_id )
            end
            @grid.remove(0)
          end          
        end
        
        def populate_grid(properties)
          properties.each do |prop|
            # 1.- the name
            @grid.add(  
              Wx::StaticText.new( self, 
                                  Wx::ID_ANY, 
                                  prop.name + ': ', 
                                  Wx::DEFAULT_POSITION,
                                  Wx::DEFAULT_SIZE,
                                  0,
                                  prop.key.to_s),
              0, 
              Wx::ALL|Wx::ALIGN_RIGHT, 
              5
            )
            # 2.- the input control
            control = nil

            # 2.1.- check simple property tipes
            case prop.value
            when String
              # TODO: detect multi-line values
              control = Wx::TextCtrl.new( self, Wx::ID_ANY, prop.value)
              evt_text(control.get_id) { |e| on_value_changed(e, prop.value) }
            when Fixnum
              control = Wx::SpinCtrl.new( self, Wx::ID_ANY, prop.value.to_s)
              evt_spinctrl(control.get_id) { |e| on_value_changed(e, prop.value)}
            when TrueClass, FalseClass
              control = Wx::CheckBox.new( self, Wx::ID_ANY)
              control.set_value(prop.value)
              evt_checkbox(control.get_id) { |e| on_value_changed(e, prop.value) }
            end
            
            # 2.2.- multiple value properties
            if (control.nil? && prop.respond_to?(:multiple))
              control = SingleColListCtrl.new( self, prop.multiple, prop.options, prop.value)
              evt_list_item_selected(control.get_id) { |e| on_value_changed(e, prop.value) }
              evt_list_item_deselected(control.get_id) { |e| on_value_changed(e, prop.value) }
            end
            
            Kernel.raise Core::Exceptions::Ui::UiArgumentError.new("Don't know how to interact with #{prop.value.class}!") if control.nil?
            @grid.add(  
              control,
              0, 
              Wx::ALL|Wx::ALIGN_LEFT|Wx::EXPAND,
              5
            )
          end
        end
        #===================================================== public methods
        public
        def initialize(parent=nil)
          super(parent, Wx::ID_ANY, Wx::DEFAULT_POSITION, Wx::Size.new(400, 400), Wx::SUNKEN_BORDER )
          self.set_min_size(Wx::Size.new(400, 400))

          box_title = Wx::StaticBox.new(self, Wx::ID_ANY, MSG[:properties])
          box = Wx::StaticBoxSizer.new(box_title, Wx::HORIZONTAL)
          @grid = Wx::FlexGridSizer.new( 2 )
          box.add(@grid, 1, Wx::ALL, 5)

          
          @apply = Wx::Button.new(self, Wx::ID_ANY, MSG[:apply])
          @apply.disable
          
          # bubble the event as evt_properties_apply
          evt_button(@apply) do |e| 
            @apply.disable
            event = Wx::CommandEvent.new(EVT_PROPERTIES_APPLY, self.get_id)
            event.set_client_data(@node_id)
            event_handler.process_event( event ) 
          end
          
          sizer = Wx::BoxSizer.new(Wx::VERTICAL)
          sizer.add(box, 1, Wx::ALL|Wx::EXPAND, 5)
          sizer.add_spacer(10)
          sizer.add(@apply, 0, Wx::ALL|Wx::ALIGN_RIGHT|Wx::ALIGN_BOTTOM, 10 )
          
          self.set_sizer(sizer)
          
          # set the default empty view
          home()

          @skip_apply = true
        end
        
        # Show the properties of a speciffic node. This widget expects the
        # properties information to be passed in an array of instances of
        # Core::Model::Support::Property and it's subclasses
        def show_properties(node_id, properties) 
          @node_id = node_id
          @skip_apply = true
          clear_grid()
          
          populate_grid(properties)

          @grid.layout
          self.get_sizer.layout
          @apply.disable
          @skip_apply = false
        end
        
        # Returns an array of objects of Core::Model::Support::Property objects
        # these can be marshalled into xml to be stored in a Note
        def get_properties()
          out = []
          label = nil
          prop = nil
          @grid.get_children.each do |item|
            if (control = item.get_window() )
              if (prop.nil? && control.instance_of?(Wx::StaticText))
                label = { :key => control.get_name, :name => control.get_label[0..-3]}
                next
              end
              case control
              when Wx::TextCtrl, Wx::SpinCtrl, Wx::CheckBox
                # key, name, value
                prop = Property.new( label[:key].to_sym, label[:name], control.get_value)
              when Wx::ListCtrl
                selections = control.get_selections#.collect do |idx| control.get_string(idx) end
                options = control.get_options #[]
                # key, name, value, options, multiple
                prop = MultipleChoiceProperty.new(
                          label[:key].to_sym, 
                          label[:name], 
                          selections, 
                          options, 
                          !control.has_flag(Wx::LC_SINGLE_SEL))
              end
              
              if !prop.nil?
                out << prop
                prop = nil
              end
            end
          end
          out
        end
        
        # not too sure about this
        def update_view(model=nil)
          clear_grid()
          home()
        end        
        
        # See Ui::PropertiesPaneInterface#update_property
        def update_property(property) 
          found = false
          i = 0
          begin
            # FIXME: ugly
            control = @grid.get_item(i)
            control = control.get_window unless control.nil?
            if control.instance_of?(Wx::StaticText)
              $logger.debug{ "StaticText control found, name: #{control.get_name}" }
              if control.get_name == property.key
                # FIXME: This only works for text & checkbox controls
                @grid.get_item(i+1).get_window.set_value( property.value )
                @apply.enable
                found = true
              end
            end
            i += 1
          end while(control)
          if not found
            Kernel.raise "The requested property :#{property.key} was not found"
          end
        end
        
        # See Ui::PropertiesPaneInterface#modified?
        def modified?
          return @apply.is_enabled()
        end
      end
    end
  end
end